unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, OmniRig_TLB, Buttons, Spin, ValueFrm, FlagFrm;

const
  HighParam = 30;
  HighValue = 5; //PM_FREQ ... PM_RITOFFSET

  ParamCodes: array[0..HighParam] of integer = (
    PM_UNKNOWN, PM_FREQ, PM_FREQA, PM_FREQB, PM_PITCH, PM_RITOFFSET, PM_RIT0,
    PM_VFOAA, PM_VFOAB, PM_VFOBA, PM_VFOBB, PM_VFOA, PM_VFOB, PM_VFOEQUAL,
    PM_VFOSWAP, PM_SPLITON, PM_SPLITOFF, PM_RITON, PM_RITOFF, PM_XITON,
    PM_XITOFF, PM_RX, PM_TX, PM_CW_U, PM_CW_L, PM_SSB_U, PM_SSB_L, PM_DIG_U,
    PM_DIG_L, PM_AM, PM_FM);

  ParamNames: array[0..HighParam] of string = (
    'UNKNOWN', 'FREQ', 'FREQA', 'FREQB', 'PITCH', 'RITOFFSET', 'RIT0',
    'VFOAA', 'VFOAB', 'VFOBA', 'VFOBB', 'VFOA', 'VFOB', 'VFOEQUAL',
    'VFOSWAP', 'SPLITON', 'SPLITOFF', 'RITON', 'RITOFF', 'XITON',
    'XITOFF', 'RX', 'TX', 'CW_U', 'CW_L', 'SSB_U', 'SSB_L', 'DIG_U',
    'DIG_L', 'AM', 'FM');


type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    ScrollBox1: TScrollBox;
    RichEdit1: TRichEdit;
    Panel4: TPanel;
    Panel5: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    SpeedButton1: TSpeedButton;
    Label3: TLabel;
    Label4: TLabel;
    Edit1: TEdit;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    SpeedButton2: TSpeedButton;
    Bevel1: TBevel;
    SpinEdit1: TSpinEdit;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Edit2: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
  private
    procedure VisibleChangeEvent(Sender: TObject);
    procedure RigTypeChangeEvent(Sender: TObject; RigNumber: Integer);
    procedure StatusChangeEvent(Sender: TObject; RigNumber: Integer);
    procedure ParamsChangeEvent(Sender: TObject; RigNumber: Integer; Params: Integer);
    procedure CustomReplyEvent(Sender: TObject; RigNumber: Integer; Command,
      Reply: OleVariant);
    procedure AddFrames;
    procedure UpdateFrames;
    function HexToSafeArray(Hex: string): Variant;
    function TextToSafeArray(Txt: string): Variant;
    function SafeArrayToHex(Arr: Variant): string;
  public
    OmniRig: TOmniRigX;
    Frames: array[1..HighParam] of TFrame;

    procedure SetFlagParam(Par: integer);
    procedure SetValueParam(Par, Value: integer);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}


function TForm1.SafeArrayToHex(Arr: Variant): string;
var
  P: PByteArray;
  i: integer;
begin
  Result := '';

  try
    if VarArrayDimCount(Arr) <> 1 then Exit;
    if VarArrayLowBound(Arr, 1) <> 0 then Exit;

    P := VarArrayLock(Arr);
    try for i:=0 to VarArrayHighBound(Arr, 1) do Result := Result + IntToHex(P[i], 2);
    finally VarArrayUnlock(Arr); end;

  except on E: Exception do
    begin RichEdit1.Lines.Add(E.Message); end;
  end;
end;


function TForm1.TextToSafeArray(Txt: string): Variant;
var
  P: PByte;
begin
  Result := VarArrayCreate([0, Length(Txt)-1], varByte);
  if Text <> '' then
    try
      P := VarArrayLock(Result);
      try Move(Txt[1], P^, Length(Txt));
      finally VarArrayUnlock(Result); end;
    except on E: Exception do
      begin RichEdit1.Lines.Add(E.Message); end;
    end;
end;


function TForm1.HexToSafeArray(Hex: string): Variant;
var
  P: PByteArray;
  i, H: integer;
begin
  H := (Length(Hex) div 2) - 1;
  Result := VarArrayCreate([0, H], varByte);

  try
    P := VarArrayLock(Result);
    try for i:=0 to H do P[i] := StrToInt('$' + Copy(Hex, i*2+1, 2));
    finally VarArrayUnlock(Result); end;

  except on E: Exception do
    begin RichEdit1.Lines.Add(E.Message); end;
  end;
end;









procedure TForm1.FormCreate(Sender: TObject);
begin
  AddFrames;

  //creating the automation object in code
  //to avoid installation on the palette
  OmniRig := TOmniRigX.Create(Self);

  OmniRig.OnVisibleChange := VisibleChangeEvent;
  OmniRig.OnRigTypeChange := RigTypeChangeEvent;
  OmniRig.OnStatusChange := StatusChangeEvent;
  OmniRig.OnParamsChange := ParamsChangeEvent;
  OmniRig.OnCustomReply := CustomReplyEvent;

  OmniRig.Connect;

  //update all indicators
  RichEdit1.Lines.Add('---- initializing client -----');
  VisibleChangeEvent(nil);
  RigTypeChangeEvent(nil, 1);
  StatusChangeEvent(nil, 1);
  ParamsChangeEvent(nil, 1, $7FFFFFFF);
  RichEdit1.Lines.Add('----- client initialized -----');
  Left := 5;
end;




//------------------------------------------------------------------------------
//                           Omni-Rig events
//------------------------------------------------------------------------------
procedure TForm1.VisibleChangeEvent(Sender: TObject);
begin
  if OmniRig.DialogVisible
    then Label1.Caption := 'Dialog VISIBLE'
    else Label1.Caption := 'Dialog invisible';

  RichEdit1.Lines.Add(Label1.Caption);
end;


procedure TForm1.RigTypeChangeEvent(Sender: TObject; RigNumber: Integer);
begin
  //we are testing only Rig1 because the two rig objects are identical
  if RigNumber <> 1 then Exit;

  Label2.Caption := 'Rig type: ' + OmniRig.Rig1.Get_RigType;
  RichEdit1.Lines.Add(Label2.Caption);
end;


procedure TForm1.ParamsChangeEvent(Sender: TObject; RigNumber,
  Params: Integer);
begin

  RichEdit1.Lines.Add('Parameters changed: ' + IntToHex(Params, 8));
  UpdateFrames;
end;


procedure TForm1.CustomReplyEvent(Sender: TObject; RigNumber: Integer;
  Command, Reply: OleVariant);
begin
  if RigNumber <> 1 then Exit;

  RichEdit1.Lines.Add(Format('Reply to custom command "%s":'#13'"%s"',
    [SafeArrayToHex(Command), SafeArrayToHex(Reply)]));
end;


procedure TForm1.StatusChangeEvent(Sender: TObject; RigNumber: Integer);
begin
  if RigNumber <> 1 then Exit;
  Label3.Caption := 'Status: ' + OmniRig.Rig1.Get_StatusStr;
  RichEdit1.Lines.Add(Label3.Caption);
  UpdateFrames;
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
  OmniRig.DialogVisible := true;
end;

procedure TForm1.SpeedButton2Click(Sender: TObject);
var
  Cmd, Trm: Variant;
begin
  if RadioButton1.Checked
    then
      begin
      Cmd := {TextToSafeArray}(Edit1.Text);
      Trm := {TextToSafeArray}(Edit2.Text);
      end
    else
      begin
      Cmd := HexToSafeArray(Edit1.Text);
      Trm := HexToSafeArray(Edit2.Text);
      end;

  OmniRig.Rig1.SendCustomCommand(Cmd, SpinEdit1.Value, Trm);
end;

procedure TForm1.AddFrames;
var
  i: integer;
begin
  for i:=1 to HighValue do
    begin
    Frames[i] := TValueFrame.Create(Self);
    with Frames[i] as TValueFrame do
      begin
      Parent := ScrollBox1;
      Name := 'X' + IntToStr(i);
      Top := 9999;
      Align := alTop;
      Label1.Caption := ParamNames[i];
      ParamCode := ParamCodes[i];
      end;
    end;

  for i:=HighValue+1 to HighParam do
    begin
    Frames[i] := TFlagFrame.Create(Self);
    with Frames[i] as TFlagFrame do
      begin
      Parent := ScrollBox1;
      Name := 'X' + IntToStr(i);
      Top := 9999;
      Align := alTop;
      Label1.Caption := ParamNames[i];
      ParamCode := ParamCodes[i];
      end;
    end;

end;


procedure TForm1.UpdateFrames;
var
  Readable, Writeable, EnabledParams: integer;
  i: integer;
begin
  Readable := OmniRig.Rig1.ReadableParams;
  Writeable := OmniRig.Rig1.WriteableParams;

  with OmniRig.Rig1 do
    EnabledParams := Vfo or Split or Rit or Xit or Tx or Mode;

  for i:=1 to HighValue do
    with Frames[i] as TValueFrame do
      begin
      if (ParamCode and Readable) = 0 then Shape1.Brush.Color := clBtnFace
      else Shape1.Brush.Color := clBlack;
      SpeedButton1.Enabled := (ParamCode and Writeable) <> 0;
      case ParamCode of
        PM_FREQ:      SpinEdit1.Value := OmniRig.Rig1.Freq;
        PM_FREQA:     SpinEdit1.Value := OmniRig.Rig1.FreqA;
        PM_FREQB:     SpinEdit1.Value := OmniRig.Rig1.FreqB;
        PM_PITCH:     SpinEdit1.Value := OmniRig.Rig1.Pitch;
        PM_RITOFFSET: SpinEdit1.Value := OmniRig.Rig1.RitOffset;
        end;
      end;

  for i:=HighValue+1 to HighParam do
    with Frames[i] as TFlagFrame do
      begin
      if (ParamCode and Readable) = 0 then Shape1.Brush.Color := clBtnFace
      else if (ParamCode and EnabledParams) <> 0 then Shape1.Brush.Color := clLime
      else Shape1.Brush.Color := clBlack;
      SpeedButton1.Enabled := (ParamCode and Writeable) <> 0;
      end;
end;


procedure TForm1.SetFlagParam(Par: integer);
begin
  case Par of
    PM_RIT0:                                   OmniRig.Rig1.ClearRit;
    PM_VFOAA, PM_VFOAB, PM_VFOBA, PM_VFOBB,
    PM_VFOA, PM_VFOB, PM_VFOEQUAL, PM_VFOSWAP: OmniRig.Rig1.Vfo := Par;
    PM_SPLITON, PM_SPLITOFF:                   OmniRig.Rig1.Split := Par;
    PM_RITON, PM_RITOFF:                       OmniRig.Rig1.Rit := Par;
    PM_XITON, PM_XITOFF:                       OmniRig.Rig1.Xit := Par;
    PM_RX, PM_TX:                              OmniRig.Rig1.Tx := Par;
    PM_CW_U, PM_CW_L, PM_SSB_U, PM_SSB_L,
    PM_DIG_U, PM_DIG_L, PM_AM, PM_FM:          OmniRig.Rig1.Mode := Par;
    end;
end;

procedure TForm1.SetValueParam(Par, Value: integer);
begin
  case Par of
    PM_FREQ:      OmniRig.Rig1.Freq := Value;
    PM_FREQA:     OmniRig.Rig1.FreqA := Value;
    PM_FREQB:     OmniRig.Rig1.FreqB := Value;
    PM_PITCH:     OmniRig.Rig1.Pitch := Value;
    PM_RITOFFSET: OmniRig.Rig1.RitOffset := Value;
    end;
end;

end.

